@@ -196,7 +196,7 @@ DMatrix cor_ar1(const DVector &rho, const DVector &wave) {
   DMatrix ans(n,n);
   for (int i = 1; i <= n; i++)
     for (int j = 1; j <= n; j++)
-      ans(i,j) = (i == j) ? 1.0 : pow(rho(1), fabs((double)(j - i)));
+      ans(i,j) = (i == j) ? 1.0 : pow(rho(1), fabs(wave(j) - wave(i)));
   return ans;
 }
 
@@ -205,8 +205,10 @@ DMatrix cor_rho_ar1(const DVector &rho, const DVector &wave) {
   DMatrix ans(n * (n - 1) / 2, 1);
   int k = 1;
   for (int i = 1; i <= n - 1; i++)
-    for (int j = i + 1; j <= n; j ++) 
-      ans(k++, 1) = (wave(j) - wave(i) == 1) ? 1.0 : (j - i) * pow(rho(1), wave(j) - wave(i) - 1);
+    for (int j = i + 1; j <= n; j ++) {
+      double tmp = fabs(wave(j) - wave(i));
+      ans(k++, 1) = (tmp == 1.0) ? 1.0 : (tmp * pow(rho(1), tmp - 1.0));
+    }
   return ans;
 }
 
